'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.withHammerManager = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _Provider = require('../Provider.js');

var _utils = require('../utils.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var ABSTRACT_METHOD = function ABSTRACT_METHOD(methodName) {
  return function () {
    throw new Error('Tried to call abstract method \'' + methodName + '\'');
  };
};

function accessAbstractMember(key) {
  var value = this[key];
  if (typeof value === 'undefined') {
    throw new Error('Every implementation of Responder needs to define \'' + key + '\' in their this scope');
  }
}

var addRecognizer = function addRecognizer(hammerManager, recognizer) {
  return hammerManager.addRecognizer.apply(hammerManager, _toConsumableArray(recognizer));
};

var removeRecognizer = function removeRecognizer(hammerManager, recognizerId) {
  return hammerManager.removeRecognizer(recognizerId);
};

var getRecognizerClass = function getRecognizerClass(instance) {
  return instance.constructor.getRecognizerClass();
};

var eventTypes = ['', 'start', 'end', 'cancel'];
var makeEvents = function makeEvents(eventId) {
  return eventTypes.map(function (type) {
    return eventId + type;
  }).join(' ');
};

var Responder = function (_React$PureComponent) {
  _inherits(Responder, _React$PureComponent);

  function Responder(props) {
    _classCallCheck(this, Responder);

    var _this = _possibleConstructorReturn(this, (Responder.__proto__ || Object.getPrototypeOf(Responder)).call(this, props));

    _this._eventInProgress = false;
    _this._didEventChange = (0, _utils.createDidChange)((0, _utils.createAreShallowEqual)(['deltaTime', 'srcEvent']), {});

    _this._getHammerManager = function () {
      return _this.props.hammerManager;
    };

    _this._handleEvent = function (event) {
      event = (0, _utils.callIfDefined)(_this.mutateEvent, event) || event;
      var exposedEvent = _this.makeExposableEvent(event);

      if (event.type.includes('start')) {
        (0, _utils.callIfDefined)(_this.props.onBegin, exposedEvent);
      }

      _this._didEventChange(event) && (0, _utils.callIfDefined)(_this.props.onEvent, exposedEvent);

      if (event.type.includes('end')) {
        (0, _utils.callIfDefined)(_this.props.onEnd, exposedEvent);
      }
    };

    _this.eventInProgress = function () {
      return _this._eventInProgress;
    };

    _this._setEventState = function (eventInProgress, event) {
      if (eventInProgress) {
        (0, _utils.callIfDefined)(_this.props.onBegin, event);
      } else {
        (0, _utils.callIfDefined)(_this.props.onEnd, event);
      }

      _this._eventInProgress = eventInProgress;
    };

    _this.getRecognizerOptions = ABSTRACT_METHOD('getRecognizerOptions');
    _this.makeExposableEvent = ABSTRACT_METHOD('makeExposableEvent');


    var isNotExtendedResponder = _this.constructor === Responder;
    (0, _invariant2.default)(!isNotExtendedResponder, 'You should never directly instantiate a Responder. ' + 'Instead extend from it to create specific responders.');
    return _this;
  }

  _createClass(Responder, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var recClass = getRecognizerClass(this);
      var recOptions = this.getRecognizerOptions();

      this.id = addRecognizer(this._getHammerManager(), [recClass, recOptions]);
      this._getHammerManager().on(makeEvents(this.id), this._handleEvent);
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      removeRecognizer(this._getHammerManager(), this.id);
    }
  }, {
    key: 'render',
    value: function render() {
      return null;
    }
  }]);

  return Responder;
}(_react2.default.PureComponent);

Responder.getRecognizerClass = ABSTRACT_METHOD('static getRecognizerClass');
exports.default = Responder;
var withHammerManager = exports.withHammerManager = function withHammerManager(WrappedComponent) {
  var Component = function Component(props) {
    return _react2.default.createElement(
      _Provider.Consumer,
      null,
      function (hammerManager) {
        return _react2.default.createElement(WrappedComponent, _extends({}, props, { hammerManager: hammerManager }));
      }
    );
  };
  Component.displayName = 'withHammerManager(' + (WrappedComponent.displayName || WrappedComponent.name) + ')';

  return Component;
};